--- /dev/null	Sun Aug 11 00:54:13 2013
+++ new/usr/src/cmd/fsdadm/fsdadm.c	Sun Aug 11 00:54:13 2013
@@ -0,0 +1,132 @@
+/*
+ * This file and its contents are supplied under the terms of the
+ * Common Development and Distribution License ("CDDL"), version 1.0.
+ * You may only use this file in accordance with the terms of version
+ * 1.0 of the CDDL.
+ *
+ * A full copy of the text of the CDDL should have accompanied this
+ * source.  A copy of the CDDL is also available via the Internet at
+ * http://www.illumos.org/license/CDDL.
+ */
+
+/*
+ * Copyright 2013 Damian Bogel.  All rights reserved.
+ */
+
+#include <libfsd.h>
+#include <stdio.h>
+#include <stdlib.h>
+
+/*
+ * Examples of usage:
+ * fsdadm -m /mnt -c 15 -l 1 200	15% read less 1 to 200 for /mnt
+ * fsdadm -n -c 50 -l 1 10	50% read less of 1 to 10ms for every new mount
+ * fsdadm -m /mnt -x		clear all disturbances for /mnt
+ * fsdadm -n -x			clear all disturbances for every new mount
+ */
+
+#define	READ_LESS	0
+#define	OPS_NUM		1
+
+int clear;
+int disable;
+int enable;
+int newmount;
+int options[OPS_NUM];
+
+int
+main(int argc, char *argv[])
+{
+	extern char *optarg;
+	extern int optind;
+
+	char *mnt = NULL;
+	int fd;
+	int opt;
+	int error;
+	fsd_stat_t stat;
+
+	while ((opt = getopt(argc, argv, "edm:c:l:xn")) != -1)
+		switch (opt) {
+		case 'e':
+			enable = 1;
+			break;
+
+		case 'd':
+			disable = 1;
+			break;
+
+		case 'c':
+			stat.fsds_read_less_chance = atoi(optarg);
+			break;
+
+		case 'm':
+			mnt = optarg;
+			break;
+
+		case 'l':
+			options[READ_LESS] = 1;
+			if (optind > argc - 1) {
+				(void) fprintf(stderr,
+				    "Error: -l requires two arguments\n");
+				return (1);
+			}
+			stat.fsds_read_less_r[0] = atoi(argv[optind-1]);
+			stat.fsds_read_less_r[1] = atoi(argv[optind]);
+			break;
+
+		case 'x':
+			clear = 1;
+			break;
+
+		case 'n':
+			newmount = 1;
+			break;
+
+		case '?':
+			(void) fprintf(stderr,
+			    "Error: Unrecognized option: "
+			    "-%c\n", optopt);
+			return (1);
+		}
+
+	fd = fsd_open();
+	if (fd == -1) {
+		(void) fprintf(stderr, "Error: Cannot open fsd device.\n");
+		return (1);
+	}
+
+	error = 0;
+	if (enable) {
+		if (fsd_enable(fd))
+			error = fsd_errno;
+	} else if (disable) {
+		if (fsd_disable(fd))
+			error = fsd_errno;
+	} else {
+		if (newmount) {
+			if (clear) {
+				if (fsd_newdisturb_off(fd))
+					error = fsd_errno;
+			} else {
+				if (fsd_newdisturb(fd, &stat))
+					error = fsd_errno;
+			}
+		} else {
+			if (clear) {
+				if (fsd_disturb_off(fd, mnt))
+					error = fsd_errno;
+			} else {
+				if (fsd_disturb(fd, mnt, &stat))
+					error = fsd_errno;
+			}
+		}
+	}
+
+	fsd_close(fd);
+	if (error) {
+		(void) fprintf(stderr, "Error: %s\n", fsd_strerr(error));
+		return (1);
+	}
+	return (0);
+}
